%%% Econ 203C (Spring 2009)
%%% Problem Set VIII, Question 1
%%% Monte Carlo Simulation of Unit Root Parameters

close all;              % close all the figures
clear all;              % clear all the variables
clc;                    % clear the screen

% set parameters
R = 1000;               % number of Monte Carlo simulation
setT = [1000,10000,40000];
                        % sample size in each simulation
rho = 1;
c = 0;
y0 = 0;
setbeta = [.5,.8,1,1.2,1.5,2];

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the estimates matrix
rho_est = [];

% Start iteration
for T = setT
    % Initialize the intermediate estimates matrix
    rho_temp = [];
    
    for r=1:R
        % Data Generating Process
        e = randn(T,1);
        y(1) = c + rho*y0 + e(1);
        for t=2:T
            y(t,1) = c + rho*y(t-1) + e(t);
        end

        % OLS estimation
        x = [y0;y(1:T-1)];
        X = [ones(T,1),x];
        pi_hat = inv(X'*X)*X'*y;

        % Stack estimates
        rho_temp = [rho_temp;pi_hat(2,1)];
    end

    % Stack estimates
    rho_est = [rho_est,rho_temp];
end

obs_setT = [];
base_setT = [];
for j = 1:length(setT)
    [obs,base] = hist((rho_est(:,j) - 1),50);
    obs_setT = [obs_setT;obs];
    base_setT = [base_setT;base];
end

for i = 1:length(setbeta)
    figure(i);
    for j = 1:length(setT)
        subplot(3,1,j);
        plot(setT(j)^setbeta(i)*base_setT(j,:),obs_setT(j,:));
        title(sprintf('T=%3d: T^{%3.1f}(rhohat-1)',setT(j),setbeta(i)));
    end 
end

figure(100);
for i = 1:length(setbeta)
    subplot(3,2,i);
    base1 = setT(1)^setbeta(i)*base_setT(1,:);
    base2 = setT(2)^setbeta(i)*base_setT(2,:);
    base3 = setT(3)^setbeta(i)*base_setT(3,:);
    plot(base1,obs_setT(1,:),'r-',base2,obs_setT(2,:),'g--',base3,obs_setT(3,:),'bx-');
    title(sprintf('T^{%3.1f}(rhohat-1)',setbeta(i)));
end
legend(sprintf('T=%3d',setT(1)),sprintf('T=%3d',setT(2)),sprintf('T=%3d',setT(3)));
