%%% Econ 203C (Spring 2009)
%%% Problem Set VIII, Question 3
%%% Monte Carlo Simulation of Subsampling in Unit Root Case

close all;              % close all the figures
clear all;              % clear all the variables
clc;                    % clear the screen

% set parameters
NS = 2000;              % number of Monte Carlo simulation
T = 100;                % sample size in each simulation
Omega = eye(2);
% Omega = [1,.5;.5,1];
c = zeros(2,1);
y0 = zeros(2,1);
y_neg1 = zeros(2,1);
Phi1(:,:,1) = [.9,0;.4,.7];
Phi2(:,:,1) = [-.2,0;.7,-.1];
Phi1(:,:,2) = [1,0;0,1];
Phi2(:,:,2) = [0,0;0,0];
Phi1(:,:,3) = [1.01,0;0,1.01];
Phi2(:,:,3) = [0,0;0,0];
% Phi1(:,:,2) = [1.5,0;.5,1.2];
% Phi2(:,:,2) = [-.5,0;.9,-.2];
% Phi1(:,:,2) = [.5,0;0,1];
% Phi2(:,:,2) = [0,0;0,0];

% get intermediate parameters
NC = size(Phi1,3);      % number of cases

% set restriction
R = [0,0,1,0,0,0,0,0,0,0;
    0,0,0,0,1,0,0,0,0,0];
% vec_pi = reshape([c';Phi1';Phi2'],10,1);
r = zeros(2,1);

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the data matrix
y = zeros(T,2);

% Initialize the estimates matrix
pi_est = [];
Wald_est = [];

for s=1:NS
    % Generate Random Numbers
    e = randn(T,2) * chol(Omega);
    
    for i=1:NC
        % Data Generating Process
        y(1,:) = c' + y0' * Phi1(:,:,i)'    + y_neg1' * Phi2(:,:,i)' + e(1,:);
        y(2,:) = c' + y(1,:) * Phi1(:,:,i)' + y0' * Phi2(:,:,i)'     + e(2,:);
        for t=3:T
            y(t,:) = c' + y(t-1,:) * Phi1(:,:,i)' + y(t-2,:) * Phi2(:,:,i)' + e(t,:);
        end

        % OLS estimation
        X = [ones(T,1),[y0';y(1:T-1,:)],[y_neg1';y0';y(1:T-2,:)]];
        pi_hat = inv(X'*X)*X'*y;
        e_hat = y - X*pi_hat;
        Omega_hat = e_hat'*e_hat/T;

        % Wald statistic
        Q_hat = X'*X/T;
        vec_pi_hat = reshape(pi_hat,10,1);
        Wald_hat = T * (R*vec_pi_hat - r)' *...
            (R*kron(Omega_hat,Q_hat^(-1))*R')^(-1) * (R*vec_pi_hat - r);

        % Stack estimates
        pi_est(s,i,:) = vec_pi_hat;
        Wald_est(s,i) = Wald_hat;
    end
end

mean(Wald_est > chi2inv(.95,2));

% Plot the figure
figure(1);
TITLE = ['Causal Case   ';'Unit Root Case';'The Other Case'];
for i=1:NC
    subplot(NC,1,i);
    [obs,base] = hist(Wald_est(:,i),50);
    plot(base,obs,'r-',base,chi2pdf(base,2)*NS*(base(2)-base(1)),'b--');
    title(TITLE(i,:));
    legend('histogram','chi(2) pdf (rescaled)');
end
