%%% Econ 203C (Spring 2009)
%%% Problem Set VIII, Question 3
%%% Monte Carlo Simulation of Subsampling in Unit Root Case

close all;              % close all the figures
clear all;              % clear all the variables
clc;                    % clear the screen

% set parameters
NS = 1000;               % number of Monte Carlo simulation
T = 100;                % sample size in each simulation
Omega = [1,.5;.5,1];
c = zeros(2,1);
y0 = zeros(2,1);
y_neg1 = zeros(2,1);
Phi1 = [1.1,0;0,1.1];
Phi2 = [0,0;0,0];

% set restriction
R = [0,0,1,0,0,0,0,0,0,0;
    0,0,0,0,1,0,0,0,0,0];
% R = [0,1,0,0,0,0,0,0,0,0;
%    0,0,0,0,0,0,1,0,0,0];
vec_pi = reshape([c';Phi1';Phi2'],10,1);
r = zeros(2,1);
% r = ones(2,1);

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the data matrix
y = zeros(T,2);

% Initialize the estimates matrix
pi_est = [];
Wald_est = [];
beta_est = [];

for s=1:NS
    % Data Generating Process
    e = randn(T,2) * chol(Omega);
    y(1,:) = c' + y0' * Phi1'    + y_neg1' * Phi2' + e(1,:);
    y(2,:) = c' + y(1,:) * Phi1' + y0' * Phi2'     + e(2,:);
    for t=3:T
        y(t,:) = c' + y(t-1,:) * Phi1' + y(t-2,:) * Phi2' + e(t,:);
    end
    
    % OLS estimation
    X = [ones(T,1),[y0';y(1:T-1,:)],[y_neg1';y0';y(1:T-2,:)]];
    pi_hat = inv(X'*X)*X'*y;
    e_hat = y - X*pi_hat;
    Omega_hat = e_hat'*e_hat/T;
    
    % Just for reference
    % simple OLS estimation of the unit root parameter
    W=[ones(T,1),[0;y(1:T-1,1)]];
    beta_hat = inv(W'*W)*W'*y(:,1);
    
    % Wald statistic
    Q_hat = X'*X/T;
    vec_pi_hat = reshape(pi_hat,10,1);
    Wald_hat = T * (R*vec_pi_hat - r)' *...
        (R*kron(Omega_hat,Q_hat^(-1))*R')^(-1) * (R*vec_pi_hat - r);
    
    % Stack estimates
    pi_est = [pi_est;vec_pi_hat'];
    Wald_est = [Wald_est;Wald_hat];
    beta_est = [beta_est;beta_hat(2)];
end

mean(Wald_est > chi2inv(.95,2))

[obs,base] = hist(Wald_est,50);
plot(base,obs,'r-',base,chi2pdf(base,2)*NS*(base(2)-base(1)),'b--');
legend('histogram','chi(2) pdf (rescaled)');
